{% extends "linkcheck/base_linkcheck.html" %}
{% load  linkcheck_coverage_tags %}

{% block extrahead %}{% endblock %}

{% block innercontent %}
    <table>
        <tr>
            <th>Model</th>
            <th>Covered</th>
            <th>Suggested config</th>
        </tr>
        {% for model in coverage_data %}
            <tr>
                <td>{{ model.name }}</td>
                <td style="font-weight: bold;color:{% if model.is_covered %}green{% else %}red{% endif %};">
                    {{ model.is_covered }}
                </td>
                <td style="font-weight: bold;color:{% if model.is_covered %}green{% else %}red{% endif %};">
                    {% if not model.is_covered %}
                        {{ model.suggested_config|render_config }}
                    {% endif %}
                </td>
            </tr>
        {% endfor %}
    </table>
{% endblock %}
