from {{ meta.app_label }}.models import {{ meta.object_name }}
<br>
class {{ meta.object_name }}Linklist(Linklist):
    model = {{ meta.object_name }}
    {% if active_field %}object_filter = {'active': True}{% endif %}
    {% if html_fields %}html_fields = [{% for f in html_fields %}'{{ f.name }}', {% endfor %}]{% endif %}
    {% if image_fields %}image_fields = [{% for f in image_fields %}'{{ f.name }}', {% endfor %}]{% endif %}
    {% if url_fields %}url_fields = [{% for f in url_fields %}'{{ f.name }}', {% endfor %}]{% endif %}
    {% if ignore_empty_fields %}ignore_empty = [{% for f in ignore_empty_fields %}'{{ f.name }}', {% endfor %}]{% endif %}
<br>
linklists = {
    '{{ meta.object_name }}': {{ meta.object_name }}Linklist,
}
<br>